/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * ibus - The Input Bus
 *
 * Copyright (c) 2007-2011 Peng Huang <shawn.p.huang@gmail.com>
 * Copyright (c) 2010-2011 Takao Fujiwara <takao.fujiwara1@gmail.com>
 * Copyright (c) 2007-2011 Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

const Clutter = imports.gi.Clutter;
const St = imports.gi.St;
const Lang = imports.lang;
const PopupMenu = imports.ui.popupMenu;
const Button = imports.ui.panelMenu.Button;
const Main = imports.ui.main;
const PopupMenuNoOpenStateChanged = imports._ibus.popupMenu.PopupMenuNoOpenStateChanged;

function SystemStatusLabelButton() {
    this._init.apply(this, arguments);
}

/* We'd like to show both icon and label. */
SystemStatusLabelButton.prototype = {
    __proto__: Button.prototype,

    _init: function(label, iconName, tooltipText) {
        Button.prototype._init.call(this, St.Align.START);
        this._init_menu();
        this._iconActor = null;
        this._iconName = null;
        this._iconActor = null;
        this._label = null;
        this._labelActor = null;
        if (label != null) {
            this.setLabel(label);
        } else {
            this.setIcon(iconName);
        }
        this.setTooltip(tooltipText);
    },

    /* The custom PopupMenu does not emit open-state-changed so that
     * the focus is kept on the text applications. */
    _init_menu: function() {
        if (this.menu) {
            this.menu.close();
        }
        this.menu = new PopupMenuNoOpenStateChanged(this.actor, St.Align.START,
                                                    St.Side.TOP, 0);
        Main.chrome.addActor(this.menu.actor, { visibleInOverview: true,
                                                affectsStruts: false });
        this.menu.actor.hide();
    },

    setIcon: function(iconName) {
        this._iconName = iconName;
        if (this._iconActor)
            this._iconActor.destroy();
        this._iconActor = new St.Icon({ icon_name: iconName,
                                        icon_type: St.IconType.SYMBOLIC,
                                        style_class: 'system-status-icon' });
        this.actor.set_child(null);
        this.actor.set_child(this._iconActor);
        this.actor.paint();
    },

    setLabel: function(label) {
        this._label = label;
        if (this._labelActor)
            this._labelActor.destroy();
        this._labelActor = new St.Label({text: label});
        this.actor.set_child(null);
        this.actor.set_child(this._labelActor);
        this.actor.paint();
    },

    setTooltip: function(text) {
        if (text != null) {
            this.tooltip = text;
            this.actor.has_tooltip = true;
            this.actor.tooltip_text = text;
        } else {
            this.actor.has_tooltip = false;
            this.tooltip = null;
        }
    }
};
