/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
/*
 * ibus - The Input Bus
 *
 * Copyright(c) 2007-2011 Peng Huang <shawn.p.huang@gmail.com>
 * Copyright(c) 2010-2011 Takao Fujiwara <takao.fujiwara1@gmail.com>
 * Copyright(c) 2007-2011 Red Hat, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or(at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA  02111-1307  USA
 */

const Gtk = imports.gi.Gtk;
const Gdk = imports.gi.Gdk;
const GObject = imports.gi.GObject;
const IBus = imports.gi.IBus;
const Lang = imports.lang;
const Signals = imports.signals;
const menu_position = imports.menu.menu_position;
const ImageMenuItem = imports.menu.ImageMenuItem;
const Menu = imports.menu.Menu;
const CheckMenuItem = imports.menu.CheckMenuItem;
const RadioMenuItem = imports.menu.RadioMenuItem;
const SeparatorMenuItem = imports.menu.SeparatorMenuItem;
const ImageShellMenuItem = imports.shellmenu.ImageShellMenuItem;
const ShellMenu = imports.shellmenu.ShellMenu;
const CheckShellMenuItem = imports.shellmenu.CheckShellMenuItem;
const RadioShellMenuItem = imports.shellmenu.RadioShellMenuItem;
const SeparatorShellMenuItem = imports.shellmenu.SeparatorShellMenuItem;
const Gettext = imports.gettext.domain('ibus10');
const _ = Gettext.gettext;

const PopupMenu = imports.ui.popupMenu;

let ICON_SIZE = Gtk.IconSize.MENU;

LanguageBar.prototype = {
    _init: function(indicator) {
        this._show = 0;
        this._enabled = false;
        this._has_focus = false;
        this._show_im_name = false;
        this._im_name = null;
        this._props = null;
        this._selected_menu_item = null;
        this._indicator = indicator;
        this._create_ui();

        this._properties = [];
        this._toplevel = new Gtk.Window.new(Gtk.WindowType.POPUP);
        this._toplevel.connect("size-allocate", this._toplevel_size_allocate_cb);
        this._toolbar = new Gtk.Toolbar();
        this._toplevel.add(this._toolbar);
        this._screen = Gdk.Screen.get_default();
        this._screen.connect("size-changed", this._screen_size_changed_cb);

        this.set_position(-1, -1);
    },

    _create_ui: function() {
        // create move handle
        /*
        this._handle = new new gtk.ToolItem();
        let handle = Handle();
        this._handle.add(handle);
        this.insert(this._handle, -1);
        handle.connect("move-end", this._handle_move_end_cb);
        */

        // create input methods menu
        // prop = IBus.Property(key = "", type = ibus.PROP_TYPE_TOGGLE, icon = "ibus", tooltip = _("Switch input method"))
        /*
        this._im_menu = new Gtk.ToggleToolButton();
        this._im_menu.set_homogeneous(false);
        this._im_menu.connect("toggled", this._im_menu_toggled_cb);
        this.insert(this._im_menu, -1);
        */

        this._about_button = new Gtk.ToolButton.new(null, Gtk.STOCK_ABOUT);
        this._about_button.set_no_show_all(true);
        this._about_button.set_tooltip_text(_("About the input method"));
        this._about_button.connect("clicked", this._about_button_clicked_cb);
        //this.insert(this._about_button, -1);
    },

    _screen_size_changed_cb: function(screen) {
        this.set_position(this._position);
    },

    _im_menu_toggled_cb: function(widget) {
        if (this._im_menu.get_active()) {
            let menu = this.emit("get-im-menu");
            menu.connect("deactivate", this._im_menu_deactivate_cb);
            menu.popup(null, null,
                menu_position,
                0,
                Gtk.get_current_event_time(),
                widget);
        }
    },

    _about_button_clicked_cb: function(widget) {
        if (this._enabled) {
            this.emit("show-engine-about");
        }
    },

    _im_menu_deactivate_cb: function(menu) {
        this._im_menu.set_active(false);
    },

    _handle_move_end_cb: function(handle) {
        x, y = this._toplevel.get_position();
        w, h = this._toplevel.get_size();
        this._position = [x + w, y + h];
        this.emit("position-changed", this._position);
    },

    _toplevel_size_allocate_cb: function(toplevel, allocation) {
        let x, y = this._position;
        let w = this._screen.get_width();
        let h = this._screen.get_height();
        if (x >= w - 80 || true) {
            this._toplevel.move(x - allocation.width, y - allocation.height);
        }
    },

    _remove_properties: function() {
        // reset all properties
        for (let i = 0; i < this._properties.length; i++) {
            this._properties[i].destroy();
        }
        this._properties = [];
    },

    _set_opacity: function(opacity) {
        if (this._toplevel.window == null) {
            this._toplevel.realize();
        }
        this._toplevel.window.set_opacity(opacity);
    },

    _replace_property: function(old_prop, new_prop) {
        old_prop.set_label(new_prop.get_label());
        old_prop.set_icon(new_prop.get_icon());
        old_prop.set_tooltip(new_prop.get_tooltip());
        old_prop.set_sensitive(new_prop.get_sensitive());
        old_prop.set_visible(new_prop.get_visible());
        old_prop.set_state(new_prop.get_state());
        old_prop.set_sub_props(new_prop.get_sub_props());
    },

    _label_select_cb: function(widget, data) {
        this._selected_menu_item = widget;
    },

    _label_deselect_cb: function(widget, data) {
        this.__selected_menu_item = null;
    },

    _label_expose_cb: function(widget, event) {
        let x = widget.allocation.x;
        let y = widget.allocation.y;
        let label = widget._prop.label.get_text();
        let gc = null;
        if (this._selected_menu_item != widget) {
            gc = widget.style.fg_gc[gtk.STATE_NORMAL];
        } else {
            gc = widget.style.bg_gc[gtk.STATE_NORMAL];
        }
        let layout = widget.create_pango_layout(label);
        widget.window.draw_layout(gc, x + 5, y + 2, layout);
    },

    _set_item_icon: function(item, prop) {
        item.set_property("always-show-image", true);
        let size = [];
        if (prop.get_icon()) {
            let width = [];
            let height = [];
            Gtk.icon_size_lookup(Gtk.IconSize.MENU, width, height);
            //item.set_image(icon.IconWidget(prop.get_icon(), width[0]));
        } else if (prop.get_label() && prop.get_label().get_text() != "") {
            //item.set_label("");
            //item.connect_after("expose-event", this._label_expose_cb);
            //item.connect("select", this._label_select_cb, null);
            //item.connect("deselect", this._label_deselect_cb, null);
        } else {
            let width = [];
            let height = [];
            Gtk.icon_size_lookup(Gtk.IconSize.MENU, width, height);
            //item.set_image(icon.IconWidget("ibus", width[0]));
        }
    },

    _on_item_property_activate: function(w, n, s) {
        this.emit("property-activate", n, s);
    },

    _on_item_show_engine_about: function(w, n, s) {
        this.emit("show-engine-about");
    },

    do_show: function() {
        this._toolbar.do_show();
    },

    do_size_request: function(requisition) {
        Gtk.Toolbar.do_size_request(this, requisition);
        this._toplevel.resize(1, 1);
    },

    set_im_icon: function(icon_name) {
        //let widget = icon.IconWidget(icon_name, 18);
        //this._im_menu.set_icon_widget(widget);
    },

    set_show_im_name: function(show) {
        //this._show_im_name = show;
        //this.set_im_name(this._im_name);
        //this._im_menu.set_is_important(show);
    },

    set_im_name: function(text) {
        this._im_name = text
        if (text) {
            //this._im_menu.set_tooltip_text(text);
            //this._im_menu.set_label(text);
        } else {
            //this._im_menu.set_tooltip_text(_("Switch input method"));
            //this._im_menu.set_label("");
        }
    },

    reset: function() {
        this._remove_properties();
    },

    set_enabled: function(enabled) {
        this._enabled = enabled;
        /*
        if (this._enabled) {
            this._about_button.show();
            this._set_opacity(1.0);
            if (this._has_focus) {
                if (this._show == 1 || this._show == 2) {
                    this.show_all();
                }
            }
        } else {
            this._about_button.hide();
            this._set_opacity(0.5);
            if (this._show == 1 || this._show == 0) {
                this.hide_all();
            }
        }
        */
    },

    is_enabled: function() {
        return this._enabled;
    },

    set_show: function(show) {
        if (show > 2) {
            show = 0;
        }
        this._show = show;
        if (this._has_focus) {
            this.focus_in();
        } else {
            this.focus_out();
        }
    },

    set_position: function(x, y) {
        let w = this._screen.get_width();
        let h = this._screen.get_height();
        let z = 0;
        if (x < 0 || y < 0) {
            x = w - 20;
            y = h - 40;
        }
        if (x > w) {
            x = w - 20;
        }
        if (y > h) {
            y = h - 40;
        }

        this._position = [x, y];
        w, h, z = this._toplevel.get_size();
        this._toplevel.move(this._position[0] - w, this._position[1] - h);
    },

    get_show: function() {
        return this._show;
    },

    register_properties: function(props) {
        this._props = props;
        if (this._show == 0) {
            return;
        }
        // refresh items on labguage bar
        this._remove_properties();
        // create new properties
        for (let i = 0; i < props.length; i++) {
            let prop = props[i];
            if (prop.type == ibus.PROP_TYPE_NORMAL) {
                item = ToolButton(prop = prop);
            } else if (prop.type == ibus.PROP_TYPE_TOGGLE) {
                item = ToggleToolButton(prop = prop);
            } else if (prop.type == ibus.PROP_TYPE_MENU) {
                item = MenuToolButton(prop = prop);
            } else if (prop.type == ibus.PROP_TYPE_SEPARATOR) {
                item = SeparatorToolItem();
            } else {
                IBusException("Unknown property type = " + prop.type);
            }

            item.connect("property-activate",
                         Lang.bind(this, this._on_item_property_activate));

            item.set_sensitive(prop.sensitive);

            item.set_no_show_all(false);

            if (prop.visible) {
                item.show();
            } else {
                item.hide();
            }

            this._properties[this._properties.length] = item;
            this.insert(item, i + 2);
        }
    },

    update_property: function(prop) {
        if (this._show == 0 && this._props) {
            for (let i = 0; this._props.get(i) != null; i++) {
                let p = this._props.get(i);
                if (p.get_key() == prop.get_key() && p.get_prop_type() == prop.get_prop_type()) {
                    this._replace_property(p, prop);
                    break;
                }
            }
        }
        for (let i = 0; i < this._properties.length; i++) {
            this._properties[i].update_property(prop);
        }
    },

    show_all: function() {
        /*
        this._toplevel.show();
        this._toplevel.window.raise();
        this._toolbar.show();
        */
    },

    hide_all: function() {
        try {
            this._toplevel.window.lower();
            this._toplevel.window.hide_all();
        } catch(e) {
        }
        let [x, y] = this._toplevel.get_position();
        this._toplevel.hide();

        // save bar position
        this._toplevel.move(x, y);
    },

    focus_in: function() {
        this._has_focus = true;
        //this._im_menu.set_sensitive(true)
        if ((this._show == 1 && this._enabled) || this._show == 2) {
            this.show_all();
        } else {
            this.hide_all()
        }
    },

    focus_out: function() {
        this._has_focus = false;
        //this._im_menu.set_sensitive(false);
        if (this._show == 2) {
            this.show_all();
        } else {
            this.hide_all();
        }
    },

    create_im_menu: function(menu) {
        if (!this._enabled) {
            return;
        }
        if (this._show != 0) {
            return;
        }
        if (!menu) {
            assert (false);
        }
        let props = this._props;
        if (!props) {
            return;
        }

        this._remove_properties();
        let item = new SeparatorMenuItem();
        item.show();
        this._properties[this._properties.length] = item;
        menu.insert(item.get_raw(), 0);

        let about_label = _("About") + " - " + this._im_name;
        let prop = new IBus.Property.new("about",
                                         IBus.PropType.NORMAL,
                                         IBus.Text.new_from_string(about_label),
                                         Gtk.STOCK_ABOUT,
                                         IBus.Text.new_from_string(_("About the Input Method")),
                                         true, true,
                                         IBus.PropState.UNCHECKED,
                                         null);
        item = new ImageMenuItem(prop = prop);
        item.set_property("always-show-image", true);
        item.set_no_show_all(true);
        item.show();
        this._properties[this._properties.length] = item;
        menu.insert(item.get_raw(), 0);
        item.connect("property-activate",
                     Lang.bind(this, this._on_item_show_engine_about));

        let i;
        for (i = 0; props.get(i) != null; i++) {
            prop = props.get(i);
        }
        let length = i;
        let radio_group = [];

        for (i = length - 1; i >= 0; i-=1) {
            prop = props.get(i);
            if (prop.get_prop_type() == IBus.PropType.NORMAL) {
                item = new ImageMenuItem(prop = prop);
                this._set_item_icon(item, prop);
            }
            else if (prop.get_prop_type() == IBus.PropType.TOGGLE) {
                item = new CheckMenuItem(prop = prop);
            }
            else if (prop.get_prop_type() == IBus.PropType.RADIO) {
                item = new RadioMenuItem(radio_group, prop = prop);
                radio_group[radio_group.length] = item;
            }
            else if (prop.get_prop_type() == IBus.PropType.SEPARATOR) {
                item = new SeparatorMenuItem();
                radio_group = null;
            }
            else if (prop.get_prop_type() == IBus.PropType.MENU) {
                item = new ImageMenuItem(prop = prop);
                this._set_item_icon(item, prop);
                let submenu = new Menu(prop);
                item.set_submenu(submenu.get_raw());
                submenu.connect("property-activate",
                                Lang.bind(this, this._on_item_property_activate));
            }
            else {
                IBusException("Unknown property type = %d" % prop.get_prop_type());
            }

            item.set_sensitive(prop.get_sensitive());

            item.set_no_show_all(true);

            if (prop.get_visible()) {
                item.show();
            } else {
                item.hide();
            }

            this._properties[this._properties.length] = item;
            menu.insert(item.get_raw(), 0);
            item.connect("property-activate",
                         Lang.bind(this, this._on_item_property_activate));
        }
    },

    create_im_menu_shell: function() {
        if (!this._enabled) {
            return;
        }
        if (this._show != 0) {
            return;
        }

        let props = this._props;
        if (!props) {
            return;
        }

        this._remove_properties();
        let item = null;
        let prop = null;
        let radio_group = [];

        for (let i = 0; props.get(i) != null; i++) {
            prop = props.get(i);
            if (prop.get_prop_type() == IBus.PropType.NORMAL) {
                item = new ImageShellMenuItem(prop = prop);
            }
            else if (prop.get_prop_type() == IBus.PropType.TOGGLE) {
                item = new CheckShellMenuItem(prop = prop);
            }
            else if (prop.get_prop_type() == IBus.PropType.RADIO) {
                item = new RadioShellMenuItem(radio_group, prop = prop);
                radio_group[radio_group.length] = item;
                for (let j = 0; j < radio_group.length; j++) {
                    radio_group[j].set_group(radio_group);
                }
            }
            else if (prop.get_prop_type() == IBus.PropType.SEPARATOR) {
                item = new SeparatorShellMenuItem();
                radio_group = null;
            }
            else if (prop.get_prop_type() == IBus.PropType.MENU) {
                let submenu = new ShellMenu(prop);
                submenu.connect("property-activate",
                                Lang.bind(this, this._on_item_property_activate));
                item = submenu;
            }
            else {
                IBusException("Unknown property type = %d" % prop.get_prop_type());
            }

            item.set_sensitive(prop.get_sensitive());

            if (prop.get_visible()) {
                item.show();
            } else {
                item.hide();
            }

            //this._properties[this._properties.length] = item;
            //menu.insert(item.get_raw(), 0);
            this._indicator.menu.addMenuItem(item.get_raw());
            item.connect("property-activate",
                         Lang.bind(this, this._on_item_property_activate));
        }

        let about_label = _("About") + " - " + this._im_name;
        prop = new IBus.Property.new("about",
                                     IBus.PropType.NORMAL,
                                     IBus.Text.new_from_string(about_label),
                                     Gtk.STOCK_ABOUT,
                                     IBus.Text.new_from_string(_("About the Input Method")),
                                     true, true,
                                     IBus.PropState.UNCHECKED,
                                     null);
        item = new ImageShellMenuItem(prop = prop);
        /*
        item.set_property("always-show-image", true);
        item.set_no_show_all(true);
        item.show();
        this._properties[this._properties.length] = item;
        */
        this._indicator.menu.addMenuItem(item.get_raw());
        item.connect("property-activate",
                     Lang.bind(this, this._on_item_show_engine_about));

        /*
        let item = new SeparatorMenuItem();
        item.show();
        this._properties[this._properties.length] = item;
        menu.insert(item.get_raw(), 0);
         */
        this._indicator.menu.addMenuItem(new SeparatorShellMenuItem().get_raw());
    },
};

function LanguageBar(indicator) {
    this._init(indicator);
}

Signals.addSignalMethods(LanguageBar.prototype);
